//---------------------------------------------------------------------------
// File: TMainForm.cpp, TDx_Play_Library, Example2 - Connections
//
// Date: 15th July, 2002
// Author: Darren John Dwyer
// Copyright: (c) 2002 Darren John Dwyer, All Rights Reserved.
//
// Description:
// Once you have created a DirectPlay interface, you need to setup a DirectPlay
// connection. Connections can be made via IPX, TCP/IP, Modem, Serial-Cable.
//
// This example shows how to enumerate DirectPlay connections are available
// using a created DirectPlay interface. It shows create a DirectPlay connection
// and destroy a DirectPlay connection.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 or 4 only
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma link "TDx_Play"
#pragma link "TDPName"
#pragma link "TDx_Play_Library_Install"
//---------------------------------------------------------------------------
#pragma link "tdpcaps"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
if (Dx_Play1) if (Dx_Play1->Created) Dx_Play1->Destroy();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::CreateButtonClick(TObject *Sender)
{
Msg("----");
Msg("Creating DirectPlay Interface");
Dx_Play1->UseAnsiString = true;
Dx_Play1->Create();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroyButtonClick(TObject *Sender)
{
Msg("----");
Msg("Destroying DirectPlay Interface");
Dx_Play1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1Create(TObject *Sender)
{
Msg("----");
Msg("DirectPlay Interface has been Created");
StatusLabel->Caption = "DirectPlay interface is currently created";
StatusLabel->Font->Color = clGreen;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1Destroy(TObject *Sender)
{
Msg("----");
Msg("DirectPlay Interface is being Destroyed");
StatusLabel->Caption = "- DirectPlay interface is not created -";
StatusLabel->Font->Color = clRed;
NumConnectionsLabel->Caption = "- no connections -";
NumConnectionsLabel->Font->Color = clRed;

// cleanup data elements
while (NumConnections>0)
	{
	if (ConnectionNames[NumConnections-1]) delete ConnectionNames[NumConnections-1];
	if (ConnectionBuffers[NumConnections-1]) delete ConnectionBuffers[NumConnections-1];
    NumConnections--;
    }

ConnectionInitialized = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1Error(TObject *Sender, AnsiString Function,
	AnsiString Error, AnsiString ErrorMessage)
{
Msg("----");
Msg("An Error Occurred in the DirectPlay Interface");
Msg("Function: "+Function);
Msg("Error: "+Error);
Msg("Error Code: "+IntToStr((int)Dx_Play1->ErrorValue));
Msg("Error Message: "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EnumConnectionsButtonClick(TObject *Sender)
{
// Enumerate available connections
ConnectionsListBox->Items->Clear();
NumConnections = 0;
NumConnectionsLabel->Caption = "- no connections -";
NumConnectionsLabel->Font->Color = clRed;
Dx_Play1->EnumConnections( NULL, NULL, 0 );
if (NumConnections>0)
	{
    NumConnectionsLabel->Font->Color = clGreen;
	NumConnectionsLabel->Caption = IntToStr((int) NumConnections) + " connections";
    }
else
	{
	NumConnectionsLabel->Font->Color = clRed;
    NumConnectionsLabel->Caption = "- no connections -";
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetCapabilitiesButtonClick(TObject *Sender)
{
CapsListBox->Items->Clear();
if (Dx_Play1->GetCaps( DPCaps1, 0 ))
    {
    CapsListBox->Items->Add( "Flags= ");
    CapsListBox->Items->AddStrings( DPCaps1->Flags_Strings );
    CapsListBox->Items->Add( "HeaderLength= "+IntToStr( (int) DPCaps1->HeaderLength ) );
    CapsListBox->Items->Add( "HundredBaud= "+IntToStr( (int) DPCaps1->HundredBaud ) );
    CapsListBox->Items->Add( "Latency= "+IntToStr( (int) DPCaps1->Latency ) );
    CapsListBox->Items->Add( "MaxBufferSize= "+IntToStr( (int) DPCaps1->MaxBufferSize ) );
    CapsListBox->Items->Add( "MaxLocalPlayer= "+IntToStr( (int) DPCaps1->MaxLocalPlayers ) );
    CapsListBox->Items->Add( "MaxPlayers= "+IntToStr( (int) DPCaps1->MaxPlayers ) );
    CapsListBox->Items->Add( "Tag= "+IntToStr( (int) DPCaps1->Tag ) );
    }
else
    CapsListBox->Items->Add( "ERROR: Could not retrieve caps" );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InitConnectionButtonClick(TObject *Sender)
{
if (ConnectionInitialized) return;
if (ConnectionsListBox->ItemIndex==-1) return;

ConnectionInitialized = Dx_Play1->InitializeConnection( ConnectionBuffers[ ConnectionsListBox->ItemIndex ] );
if (ConnectionInitialized)
	{
	ConnectionInitializedLabel->Caption = "connection established";
    ConnectionInitializedLabel->Font->Color = clGreen;
    }
else
	{
	ConnectionInitializedLabel->Caption = "- not connected -";
    ConnectionInitializedLabel->Font->Color = clRed;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1EnumConnections(TObject *Sender,
	const GUID *Guid, void *Connection, DWORD ConnectionSize, TDPName *Name,
	DWORD Flags, void *Context, bool &Continue)
{
if (NumConnections<MAX_CONNECTIONS)
    {
    // save information for later use
    ConnectionGuids[NumConnections] = *Guid;
    ConnectionBuffers[NumConnections] = new byte[ConnectionSize];
    if (ConnectionBuffers[NumConnections])
       CopyMemory( ConnectionBuffers[NumConnections], Connection, ConnectionSize );

    ConnectionSizes[NumConnections] = ConnectionSize;
    ConnectionNames[NumConnections] = new TDPName(this);
    if (ConnectionNames[NumConnections])
    	{
    	ConnectionNames[NumConnections]->LongNameA = Name->LongNameA;
    	ConnectionNames[NumConnections]->ShortNameA = Name->ShortNameA;
        }
    ConnectionFlags[NumConnections] = Flags;

    // update the form's display
	ConnectionsListBox->Items->Add( IntToStr((int) NumConnections)+ ": "+Name->ShortNameA );
	NumConnections++;
	}
else
	Continue = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CloseConnectionButtonClick(TObject *Sender)
{
if (ConnectionInitialized)
	{
	// simply destroy the Dx_Play component to close the connection
	if (Dx_Play1->Created) Dx_Play1->Destroy();

    CapsListBox->Items->Clear();
	ConnectionInitializedLabel->Caption = "- not connected -";
    ConnectionInitializedLabel->Font->Color = clRed;
    }
}
//---------------------------------------------------------------------------
